#include "Wire.h"
#include <Adafruit_NeoPixel.h>

#define PINE 8

#define DS3231_I2C_ADDRESS 0x68

Adafruit_NeoPixel strip8 = Adafruit_NeoPixel(78 , PINE, NEO_GRB + NEO_KHZ800);

int red, green, blue, pastto, j = 0;
//different variables used in showing time

int color[28][3] = {
  {0, 128, 128}, {171, 62, 91}, {203, 230, 91},
  {179, 120, 211}, {0, 255, 255}, {139, 69, 19},
  {255, 0, 98}, {179, 204, 87}, {239, 119, 111},
  {255, 0, 255}, {226, 247, 206}, {182, 216, 192},
  {209, 30, 72}, {46, 139, 89}, {30, 164, 157},
  {254, 97, 150}, {89, 240, 220}, {100, 149, 239},
  {131, 176, 145}, {138, 43, 226}, {128, 128, 128},
  {255, 78, 80}, {179, 120, 211}, {244, 221, 81},
  {171, 62, 91}, {0, 255, 255}, {236, 240, 129}, {255, 69, 0}
};  //2D array holding values of 28 colors

int minIndex[11][2] = {
  {13, 14}, {15, 16}, {22, 27},
  {17, 21}, {2, 8}, {9, 12},
  {2, 8}, {17, 21}, {22, 27},
  {15, 16}, {13, 14}
};

int hourIndex[24][2] = {
                        {67, 71}, {38, 39}, {40, 42}, {46, 48},
                        {43, 45}, {49, 51}, {52, 53}, {54, 57},
                        {60, 62}, {58, 60}, {72, 73}, {63, 66},
                        {67, 71}, {38, 39}, {40, 42}, {46, 48},
                        {43, 45}, {49, 51}, {52, 53}, {54, 57},
                        {60, 62}, {58, 59}, {72, 73}, {63, 66}
};

byte decToBcd(byte val)   //for RTC
{
  return ( (val / 10 * 16) + (val % 10) );
}

byte bcdToDec(byte val)   //for RTC
{
  return ( (val / 16 * 10) + (val % 16) );
}

void setup()
{
  Wire.begin();
  strip8.begin();
  strip8.show();
  strip8.setBrightness(255);
}

void readDS3231time(byte *second,
                    byte *minute,
                    byte *hour,
                    byte *dayOfWeek,
                    byte *dayOfMonth,
                    byte *month,
                    byte *year)   //for RTC
{
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0); // set DS3231 register pointer to 00h
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDRESS, 7);
  // request seven bytes of data from DS3231 starting from register 00h
  *second = bcdToDec(Wire.read() & 0x7f);
  *minute = bcdToDec(Wire.read());
  *hour = bcdToDec(Wire.read() & 0x3f);
  *dayOfWeek = bcdToDec(Wire.read());
  *dayOfMonth = bcdToDec(Wire.read());
  *month = bcdToDec(Wire.read());
  *year = bcdToDec(Wire.read());
}

void glowTime()   //showing time
{
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
  readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month,
                 &year);

  if (minute % 5 == 0 && second == 01)  //turning off the previously lit LEDs
  {
    for (int i = 0; i <= 78; i++)
    {
      strip8.setPixelColor(i, 0, 0, 0);
      strip8.show();

    }
    delay(1000);
  }

  red = color[hour][0];
  green = color[hour][1];
  blue = color[hour][2];

  if (minute < 5){
    //It Is
  for (int i = 0; i <= 1; i++) {
    strip8.setPixelColor(i, red, green , blue);
  }
  //O'Clock
  for (int i = 74; i <= 77; i++) {
    strip8.setPixelColor(i, red, green , blue);
  }
  
  for (int i = hourIndex[hour][0]; i <= hourIndex[hour][1]; i++){
   strip8.setPixelColor(i, red, green, blue); 
  }
  }
  
  else{
  //It Is
  for (int i = 0; i <= 1; i++) {
    strip8.setPixelColor(i, red, green , blue);
  }
  //O'Clock
  for (int i = 74; i <= 77; i++) {
    strip8.setPixelColor(i, red, green , blue);
  }
  //Either Past or To
  if (minute < 35) {
    for (int i = 33; i <= 35; i++) {
      strip8.setPixelColor(i, red, green, blue);
    }
  }
  else {
    for (int i = 36; i <= 37; i++) {
      strip8.setPixelColor(i, red, green, blue);
    }
  }
  //Minutes
  for (int i = 28; i <= 32; i++){
    strip8.setPixelColor(i, red, green, blue);
  }

  //Actual Hours
  if (minute < 35){
   for (int i = hourIndex[hour][0]; i <= hourIndex[hour][1]; i++){
    strip8.setPixelColor(i, red, green, blue);
  }
  }
  else{
   for (int i = hourIndex[hour + 1][0]; i <= hourIndex[hour + 1][1]; i++){
    strip8.setPixelColor(i, red, green, blue);
  }
  }
  //Actual Minutes
  if (minute >=5 && minute < 10)  j = 0;
  else if (minute >= 10 && minute < 15) j = 1;
  else if (minute >= 15 && minute < 20) j = 2;
  else if (minute >= 20 && minute < 25) j = 3;
  else if (minute >= 25 && minute < 30) j = 4;
  else if (minute >= 30 && minute < 35) j = 5;
  else if (minute >= 35 && minute < 40) j = 6;
  else if (minute >= 40 && minute < 45) j = 7;
  else if (minute >= 45 && minute < 50) j = 8;
  else if (minute >= 50 && minute < 55) j = 9;
  else j = 10;
  for (int i = minIndex[j][0]; i <= minIndex[j][1]; i++){
    strip8.setPixelColor(i, red, green, blue);
  }
  }
  strip8.show();
}

void loop()
{
  /*byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
  // retrieve data from DS3231
  readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month,
                 &year);
  */
  glowTime();
}
